package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.shipment.convert.BoxAirCheckoutConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirCheckoutDO;
import cn.iocoder.yudao.module.shipment.service.boxAirCheckout.BoxAirCheckoutService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_AIR_CHECKOUT_NUM_ERROR;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_AIR_CHECKOUT_REPEAT_COMMIT;

@Validated
@RestController
@Api(tags = "管理后台 - 空运出仓")
@RequestMapping("/ecw/box-air-checkout")
public class BoxAirCheckoutController {

    @Resource
    private BoxAirCheckoutService boxAirCheckoutService;
    @Resource
    private RedisHelper redisHelper;

    @Resource
    private BoxMergePkgService boxMergePkgService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private OrderService orderService;

    /**
     * 出仓操作的防重复提交缓存key
     */
    private static final String BOX_AIR_CHECKOUT_KEY = "jiedao:box:air:checkout:{0}";
    @PostMapping("/create")
    @ApiOperation("创建空运出仓")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:create')")
    public CommonResult<Long> createBoxAirCheckout(@Valid @RequestBody BoxAirCheckoutCreateReqVO createReqVO) {
        String redisKey = MessageFormat.format(BOX_AIR_CHECKOUT_KEY, createReqVO.getShipmentId().toString());
        if (createReqVO.getPlanNum().intValue() != createReqVO.getRealNum().intValue()) {
            return error(BOX_AIR_CHECKOUT_NUM_ERROR);
        }
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1){
            return error(BOX_AIR_CHECKOUT_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 2, TimeUnit.MINUTES);
        Long c = boxAirCheckoutService.createBoxAirCheckout(createReqVO);
        redisHelper.delete(redisKey);
        return success(c);
    }

    @GetMapping("/confirmAirCheckout")
    @ApiOperation("确认出仓")
    @ApiImplicitParam(name = "shipmentId",value = "出货id",required = true,dataTypeClass = Long.class)
    public CommonResult<Boolean> confirmAirCheckout(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxAirCheckoutService.confirmAirCheckout(shipmentId));
    }

    @PutMapping("/update")
    @ApiOperation("更新空运出仓")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:update')")
    public CommonResult<Boolean> updateBoxAirCheckout(@Valid @RequestBody BoxAirCheckoutUpdateReqVO updateReqVO) {
        boxAirCheckoutService.updateBoxAirCheckout(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除空运出仓")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:delete')")
    public CommonResult<Boolean> deleteBoxAirCheckout(@RequestParam("id") Long id) {
        boxAirCheckoutService.deleteBoxAirCheckout(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得空运出仓")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:query')")
    public CommonResult<BoxAirCheckoutBackVO> getBoxAirCheckout(@RequestParam("id") Long id) {
        BoxAirCheckoutDO boxAirCheckout = boxAirCheckoutService.getBoxAirCheckout(id);
        return success(BoxAirCheckoutConvert.INSTANCE.convert(boxAirCheckout));
    }

    @GetMapping("/list")
    @ApiOperation("获得空运出仓列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:query')")
    public CommonResult<List<BoxAirCheckoutBackVO>> getBoxAirCheckoutList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxAirCheckoutDO> list = boxAirCheckoutService.getBoxAirCheckoutList(ids);
        return success(BoxAirCheckoutConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得空运出仓分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:query')")
    public CommonResult<PageResult<BoxAirCheckoutBackVO>> getBoxAirCheckoutPage(@Valid BoxAirCheckoutQueryVO query, PageVO page) {
        PageResult<BoxAirCheckoutDO> pageResult = boxAirCheckoutService.getBoxAirCheckoutPage(query, page);
        return success(BoxAirCheckoutConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出空运出仓 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-checkout:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxAirCheckoutExcel(@Valid BoxAirCheckoutQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxAirCheckoutDO> list = boxAirCheckoutService.getBoxAirCheckoutList(query);
        // 导出 Excel
        List<BoxAirCheckoutBackVO> datas = BoxAirCheckoutConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "空运出仓.xls", "数据", BoxAirCheckoutBackVO.class, datas);
    }

    @GetMapping("/getBoxAirCheckout4PDAPage")
    @ApiOperation("PDA - 获得空运出仓分页")
    public CommonResult<PageResult<BoxAirCheckoutPDAVO>> getBoxAirCheckoutPage(@RequestParam(value = "selfNo",required = false) String selfNo, PageVO page) {
        PageResult<BoxAirCheckoutPDAVO> pageResult = boxAirCheckoutService.getBoxAirCheckout4PDAPage(selfNo, page);
        return success(pageResult);
    }

    @GetMapping("/getAirCheckoutByShipmentId")
    @ApiOperation("根据出货id获得空运出仓")
    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-shipment:query')")
    public CommonResult<BoxAirCheckoutBackVO> getAirCheckoutByShipmentId(@RequestParam("shipmentId") Long shipmentId) {
        BoxAirCheckoutDO boxAirCheckoutDO = boxAirCheckoutService.getAirCheckoutByShipmentId(shipmentId);
        if (boxAirCheckoutDO == null) {
            boxAirCheckoutDO = new BoxAirCheckoutDO();
        }
        Integer planNum = boxAirCheckoutService.getPlanNum(shipmentId);
        Integer realNum = boxAirCheckoutService.getRealNum(shipmentId);
        boxAirCheckoutDO.setPlanNum(planNum);
        boxAirCheckoutDO.setRealNum(realNum);
        return success(BoxAirCheckoutConvert.INSTANCE.convert(boxAirCheckoutDO));
    }
}
