package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.FutureBoxConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.FutureBoxDO;
import cn.iocoder.yudao.module.shipment.service.futureBox.FutureBoxService;
import cn.iocoder.yudao.module.shipment.vo.futureBox.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 预定舱计划")
@RequestMapping("/ecw/future-box")
public class FutureBoxController {

    @Resource
    private FutureBoxService futureBoxService;

    @PostMapping("/create")
    @ApiOperation("创建预定舱计划")
    @PreAuthorize("@ss.hasPermission('ecw:future-box:create')")
    public CommonResult<Long> createFutureBox(@Valid @RequestBody FutureBoxCreateReqVO createReqVO) {
        return success(futureBoxService.createFutureBox(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新预定舱计划")
    @PreAuthorize("@ss.hasPermission('ecw:future-box:update')")
    public CommonResult<Boolean> updateFutureBox(@Valid @RequestBody FutureBoxUpdateReqVO updateReqVO) {
        futureBoxService.updateFutureBox(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除预定舱计划")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:future-box:delete')")
    public CommonResult<Boolean> deleteFutureBox(@RequestParam("id") Long id) {
        futureBoxService.deleteFutureBox(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得预定舱计划")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:future-box:query')")
    public CommonResult<FutureBoxBackVO> getFutureBox(@RequestParam("id") Long id) {
        FutureBoxDO futureBox = futureBoxService.getFutureBox(id);
        return success(FutureBoxConvert.INSTANCE.convert(futureBox));
    }

    @GetMapping("/list")
    @ApiOperation("获得预定舱计划列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:future-box:query')")
    public CommonResult<List<FutureBoxBackVO>> getFutureBoxList(@RequestParam("ids") Collection<Long> ids) {
        List<FutureBoxDO> list = futureBoxService.getFutureBoxList(ids);
        return success(FutureBoxConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得预定舱计划分页")
    @PreAuthorize("@ss.hasPermission('ecw:future-box:query')")
    public CommonResult<PageResult<FutureBoxBackVO>> getFutureBoxPage(@Valid FutureBoxQueryVO query, PageVO page) {
        page.setPage(query.getPageNo());
        page.setRows(query.getPageSize());
        return success(futureBoxService.getFutureBoxBackPage(query, page));
    }

    @GetMapping("/getToBeWareHousedVolume")
    @ApiOperation("根据条件得到待入仓方数/重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getToBeWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        BigDecimal data = futureBoxService.getToBeWareHousedVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getWareHousedVolume")
    @ApiOperation("根据条件得到已入仓方数/重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        BigDecimal data = futureBoxService.getWareHousedVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getBoxedVolume")
    @ApiOperation("根据条件得到已封柜方数/出仓重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getBoxedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        BigDecimal data = futureBoxService.getBoxedVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getBoxedWeight")
    @ApiOperation("根据条件得到已封柜方数/出仓重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getBoxedWeight(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        BigDecimal data = futureBoxService.getBoxedWeight(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getExceptionVolume")
    @ApiOperation("根据条件得到异常订单方数/重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getExceptionVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo){
        BigDecimal data = futureBoxService.getExceptionVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getOrderReportVolume")
    @ApiOperation("根据条件得到自单代报订单方数/重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getOrderReportVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo){
        BigDecimal data = futureBoxService.getOrderReportVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }

    @GetMapping("/getWarehousingVolume")
    @ApiOperation("根据条件得到待排单订单方数/重量(海运方数、空运重量)")
    public CommonResult<BigDecimal> getWarehousingVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo){
        BigDecimal data = futureBoxService.getWarehousingVolume(futureBoxTotalQueryVo);
        return success(null == data ? BigDecimal.ZERO : data);
    }


}
